@@ -5,17 +5,17 @@ import re
 import numpy as np
 import pandas as pd
 import tensorflow as tf
-
 import transformers
+from tokenizers import BertWordPieceTokenizer
+
 from kaggle_runner import may_debug
 from kaggle_runner.defaults import DEBUG
 from kaggle_runner.utils.kernel_utils import (get_kaggle_dataset_input,
                                               get_obj_or_dump)
 from kaggle_runner.utils.tpu import BATCH_SIZE
-from tokenizers import BertWordPieceTokenizer
 
-if DEBUG:
-    tf.executing_eagerly()
+#if DEBUG:
+tf.executing_eagerly()
 # Dataloading related
 AUTO = tf.data.experimental.AUTOTUNE
 
@@ -125,12 +125,12 @@ TEST_LEN  = len(x_test)
 if DEBUG:
     TRAIN_LEN = TRAIN_LEN//10
     VALID_LEN = VALID_LEN//10
-    TEST_LEN = TEST_LEN//100
     x_train = x_train[:TRAIN_LEN, :140]
     y_train = y_train[:TRAIN_LEN]  # y just all pass, they are labels
     x_valid = x_valid[:VALID_LEN, :140]
     y_valid = y_valid[:VALID_LEN] # it just one dimention
-    x_test  = x_test[:TEST_LEN, :140]
+    # TEST_LEN = TEST_LEN//100
+    # x_test  = x_test[:TEST_LEN, :140]
 
 
 # +
