@@ -190,7 +190,7 @@ color_my_prompt () {
     local __cur_location="\[\033[01;34m\]\w"
     local __git_branch_color="\[\033[31m\]"
     # local __git_branch="\`ruby -e \"print (%x{git branch 2> /dev/null}.grep(/^\*/).first || '').gsub(/^\* (.+)$/, '(\1) ')\"\`"
-    local __git_branch='`git branch 2> /dev/null | grep -e ^* | sed -E  s/^\\\\\*\ \(.+\)$/\(\\\\\1\)\ /`'
+    local __git_branch='`git branch 2> /dev/null | grep -e ^* | ${SED:-sed} -E  s/^\\\\\*\ \(.+\)$/\(\\\\\1\)\ /`'
     local __prompt_tail="\[\033[35m\]$"
     local __last_color="\[\033[00m\]"
     export PS1="$__user_and_host $__cur_location $__git_branch_color$__git_branch$__prompt_tail$__last_color "
@@ -209,7 +209,8 @@ OLDPWD=/root
 # color_my_prompt
 locale-gen
 echo "#" $(grep 'cpu ' /proc/stat >/dev/null;sleep 0.1;grep 'cpu ' /proc/stat | awk -v RS="" '{print "CPU: "($13-$2+$15-$4)*100/($13-$2+$15-$4+$16-$5)"%"}') "Mem: "$(awk '/MemTotal/{t=$2}/MemAvailable/{a=$2}END{print 100-100*a/t"%"}' /proc/meminfo) "Uptime: "$(uptime | awk '{print $1 " " $2 " " $3}')
-echo "#" $TPU_NAME
+echo "#" TPU_NAME=$TPU_NAME
+nvidia-smi
 conda activate base
 EOF
 }
@@ -361,7 +362,8 @@ fi
 
 if [ x"${PHASE}" = x"data" ]; then
     bash ./rvs.sh $SERVER $PORT >/dev/null & make m & # just keep one rvs incase
-    make
+    make dataset
+    wait
 fi
 
 if [ x"${PHASE}" = x"run" ]; then
