@@ -10,48 +10,30 @@ import tensorflow as tf
 import tensorflow.keras.backend as K
 from IPython.core.debugger import set_trace
 from sklearn.model_selection import KFold
-from tensorflow.keras.callbacks import (
-    EarlyStopping,
-    LearningRateScheduler,
-    ModelCheckpoint,
-)
-from tensorflow.keras.layers import (
-    Activation,
-    BatchNormalization,
-    Bidirectional,
-    CuDNNLSTM,
-    Dense,
-    Embedding,
-    GlobalAveragePooling1D,
-    GlobalMaxPooling1D,
-    Input,
-    Lambda,
-    Layer,
-    PReLU,
-    SpatialDropout1D,
-    add,
-    concatenate,
-)
+from tensorflow.keras.callbacks import (EarlyStopping, LearningRateScheduler,
+                                        ModelCheckpoint)
+from tensorflow.keras.layers import (LSTM, Activation, BatchNormalization,
+                                     Bidirectional, Dense, Embedding,
+                                     GlobalAveragePooling1D,
+                                     GlobalMaxPooling1D, Input, Lambda, Layer,
+                                     PReLU, SpatialDropout1D, add, concatenate)
 from tensorflow.keras.metrics import binary_crossentropy, mean_absolute_error
 from tensorflow.keras.models import Model, load_model
 from tensorflow.keras.optimizers import Adam
 from tensorflow.python.ops import math_ops
 
-import data_prepare as d
-import utils
-
+import kaggle_runner.datasets.data_prepare as d
+import kaggle_runner.utils.kernel_utils as utils
+from kaggle_runner.defaults import *
+from kaggle_runner.logs import NBatchProgBarLogger
 # from gradient_reversal_keras_tf.flipGradientTF import GradientReversal
-from kaggle_runner.losses import (
-    binary_crossentropy_with_focal,
-    binary_crossentropy_with_focal_seasoned,
-)
-from kaggle_runner.metrics.metrics import (
-    binary_sensitivity,
-    binary_sensitivity_np,
-    binary_specificity,
-)
-from kaggle_runner.modules import reinitLayers
+from kaggle_runner.losses import (binary_crossentropy_with_focal,
+                                  binary_crossentropy_with_focal_seasoned)
+from kaggle_runner.metrics.metrics import (binary_sensitivity,
+                                           binary_sensitivity_np,
+                                           binary_specificity)
 from kaggle_runner.modules.attention import AttentionRaffel
+from kaggle_runner.utils.kernel_utils import reinitLayers
 
 NUM_MODELS = 2  # might be helpful but...
 
@@ -83,15 +65,18 @@ RESTART_TRAIN = False
 RESTART_TRAIN_RES = True
 RESTART_TRAIN_ID = False
 
-NO_AUX = False
+NO_AUX = True
 Y_TRAIN_BIN = False  # with True, slightly worse
 
-FOCAL_LOSS = True
+FOCAL_LOSS = False
 
 PRD_ONLY = True  # will not train the model
 NOT_PRD = True
-FINAL_SUBMIT = False
+
+FOCAL_LOSS_GAMMA = 0.0
+FINAL_SUBMIT = True
 FINAL_DEBUG = True
+
 if FINAL_SUBMIT:
     TARGET_RUN = "lstm"
     EPOCHS = 6
@@ -150,7 +135,7 @@ class KaggleKernel:
         if PRD_ONLY or TARGET_RUN_READ_RESULT or ANA_RESULT:
             pass
         else:  # keep record training parameters
-            self.emb.dump_obj(
+            utils.dump_obj(
                 f"Start run with FL_{FOCAL_LOSS}_{FOCAL_LOSS_GAMMA}_{ALPHA} lr {STARTER_LEARNING_RATE}, decay {LEARNING_RATE_DECAY_PER_EPOCH}, \
 BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_TRAIN_BIN {Y_TRAIN_BIN}",
                 "run_info.txt",
@@ -166,6 +151,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
         ) = self.emb.data_prepare(action)
         # picked ones are binarized
         self.train_y_aux_all = self.train_df[d.AUX_COLUMNS].values
+
         if Y_TRAIN_BIN:
             self.train_y_float_backup = self.train_y_all
             self.train_y_all = np.where(self.train_y_all >= 0.5, True, False)
@@ -241,113 +227,6 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
     #    return m
 
     # so nine model for them...
-    def build_identity_model(self, identity_out_num):
-        # d.IDENTITY_COLUMNS  # with more than 500
-        """build lstm model
-
-        :param embedding_matrix:
-        :param num_aux_targets:
-        :return: the built model lstm
-        """
-        words = Input(shape=(None,))
-        x = Embedding(
-            *self.embedding_matrix.shape,
-            weights=[self.embedding_matrix],
-            trainable=False,
-        )(words)
-        x = SpatialDropout1D(0.2)(x)
-        x = Bidirectional(
-            CuDNNLSTM(int(LSTM_UNITS // 2), return_sequences=True))(x)
-
-        hidden = concatenate(
-            [GlobalMaxPooling1D()(x), GlobalAveragePooling1D()(x), ])
-        hidden = add(
-            [hidden, Dense(int(DENSE_HIDDEN_UNITS // 2),
-                           activation="relu")(hidden)]
-        )
-        result = Dense(identity_out_num, activation="sigmoid")(hidden)
-        # result_with_aux = Dense(num_aux_targets+1, activation='sigmoid')(hidden)
-
-        model = Model(inputs=words, outputs=result)
-        # model = Model(inputs=words, outputs=result_with_aux)
-        # model.compile(loss='binary_crossentropy', optimizer='adam')
-        try:
-            # need to improve#for target it should be fine, they are positive related
-            model.compile(
-                loss="binary_crossentropy",
-                optimizer="adam",
-                metrics=[mean_absolute_error,
-                         binary_sensitivity, binary_specificity],
-            )
-        except Exception as e:
-            logger.warning(e)
-            # need to improve#for target it should be fine, they are positive related
-            model.compile(
-                loss="binary_crossentropy",
-                optimizer="adam",
-                metrics=[mean_absolute_error],
-            )
-
-        return model
-
-    def run_identity_model(self, subgroup, features, labels, params):
-        set_trace()
-
-        prefix = params["prefix"]
-        file_name = f"{prefix}_{subgroup}_0.hdf5"
-        params["check_point_path"] = file_name
-        pred_only = params.get("predict_only", False)
-        if not pred_only:
-            target_subgroup = labels[:, d.IDENTITY_COLUMNS.index(subgroup)]
-
-        if RESTART_TRAIN_ID or not os.path.isfile(file_name):
-            if pred_only:
-                raise RuntimeError("Need to have a trained model to predict")
-            model = self.build_identity_model(1)  # one model per identity...
-            self.run_model_train(model, features, target_subgroup, params)
-        else:
-            logger.info("restore from the model file")
-            model = load_model(
-                file_name,
-                custom_objects={
-                    "AttentionRaffel": AttentionRaffel,
-                    "binary_sensitivity": binary_sensitivity,
-                    "binary_specificity": binary_specificity,
-                },
-            )
-
-            # TODO change logger
-            logger.info(
-                f"restore from the model file {file_name} -> done\n\n\n\n")
-
-            continue_train = params.get("continue_train", False)
-            if continue_train:
-                params["starter_lr"] = params["starter_lr"] / 16
-                logger.debug(f"continue train with learning rate /16")
-                self.run_model_train(model, features, target_subgroup, params)
-
-        if not pred_only:
-            identity_predict = self.run_model(model, "predict", self.train_X)
-            identity_predict_for_metric = identity_predict[self.identity_idx]
-
-            s = binary_sensitivity_np(
-                identity_predict_for_metric.reshape(-1), target_subgroup
-            )
-            logger.info(f"for {subgroup}, predict_sensitivity is {s}")
-
-            predict_file_name = f"{prefix}_{subgroup}_pred.pkl"
-            id_preds = {}
-            id_preds[subgroup] = identity_predict
-            pickle.dump(identity_predict, open(predict_file_name, "wb"))
-        else:  # only predict for test
-            predict = self.run_model(model, "predict", features)
-            logger.debug(
-                f"in test set, predicted {subgroup}: mean {predict.mean()}, cnt {(predict>=0.5).sum()}, all_cnt {len(predict)}"
-            )
-            # self._val_index = val_ind  # for debug
-            # pred = model.predict(self.train_X[val_ind], verbose=2)
-            # self.oof_preds[val_ind] += pred
-        # pickle.dump(self.oof_preds, open(prefix + "predicts", 'wb'))
 
     def build_lstm_model_customed(
         self,
@@ -370,13 +249,16 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
         logger.debug(
             f"model detail: loss {loss}, hidden_act {hidden_act}, with_BN {with_BN}"
         )
+
         if num_aux_targets > 0 and not with_aux:
             raise RuntimeError(
                 "aux features numbers given but aux not enabled")
+
         if num_aux_targets <= 0 and with_aux:
             raise RuntimeError("aux features numbers invalid when aux enabled")
 
         words = Input(shape=(d.MAX_LEN,))  # (None, 180)
+
         if embedding_layer is not None:
             x = embedding_layer(words)
         else:
@@ -385,8 +267,8 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
             )(words)
 
         x = SpatialDropout1D(0.2)(x)
-        x = Bidirectional(CuDNNLSTM(LSTM_UNITS, return_sequences=True))(x)
-        x = Bidirectional(CuDNNLSTM(LSTM_UNITS, return_sequences=True))(x)
+        x = Bidirectional(LSTM(LSTM_UNITS, return_sequences=True))(x)
+        x = Bidirectional(LSTM(LSTM_UNITS, return_sequences=True))(x)
 
         hidden = concatenate(
             [
@@ -397,10 +279,12 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
         )
 
         activate_type = hidden_act
+
         if activate_type == "prelu":  # found it not working
             hidden = add(
                 [hidden, PReLU()(Dense(DENSE_HIDDEN_UNITS, activation=None)(hidden))]
             )
+
             if with_BN:
                 hidden = BatchNormalization()(hidden)
             hidden = add(
@@ -411,6 +295,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
                 [hidden, Dense(DENSE_HIDDEN_UNITS,
                                activation=activate_type)(hidden)]
             )
+
             if with_BN:
                 hidden = BatchNormalization()(hidden)
             hidden = add(
@@ -454,8 +339,8 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
         )
 
         x = SpatialDropout1D(0.2)(x)
-        x = Bidirectional(CuDNNLSTM(LSTM_UNITS, return_sequences=True))(x)
-        x = Bidirectional(CuDNNLSTM(LSTM_UNITS, return_sequences=True))(x)
+        x = Bidirectional(LSTM(LSTM_UNITS, return_sequences=True))(x)
+        x = Bidirectional(LSTM(LSTM_UNITS, return_sequences=True))(x)
 
         hidden = concatenate(
             [GlobalMaxPooling1D()(x), GlobalAveragePooling1D()(x), ])
@@ -491,20 +376,23 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
         # + , - infinity, we cannot use least square error, we use maximum likelihood, the line
         # can still be imagined to there. for every w guess, we have a log-likelihood for the
         # line. we need to find the ML line
+
         return model
 
     def run_lstm_model(
-        self,
-        final_train=False,
-        n_splits=NUM_MODELS,
-        predict_ones_with_identity=True,
-        params=None,
-        val_mask=None,
+            self,
+            final_train=False,
+            n_splits=NUM_MODELS,
+            predict_ones_with_identity=True,
+            params=None,
+            val_mask=None,
     ):
         # TODO condiser use train test split as
         """
-        train_df, validate_df = model_selection.train_test_split(train, test_size=0.2)
-        logger.info('%d train comments, %d validate comments' % (len(train_df), len(validate_df)))"""
+        train_df, validate_df = model_selection.train_test_split(train,
+        test_size=0.2)
+        logger.info('%d train comments, %d validate comments' % (len(train_df),
+        len(validate_df)))"""
 
         # self.oof_preds = np.zeros((self.train_X.shape[0], 1 + self.train_y_aux.shape[1]))
         test_preds = np.zeros((self.to_predict_X_all.shape[0]))
@@ -528,12 +416,15 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
             train_X, train_y = train_data
             train_y_aux = train_y_aux_passed
             val_y_aux = val_y_aux_passed
+
         if train_y_aux is None and not NO_AUX:
             raise RuntimeError("Need aux labels to train")
+
         if val_y_aux is None and not NO_AUX:
             raise RuntimeError("Need aux labels to validate")
 
         val_data = params.get("val_data")
+
         if val_data is None:  # used to dev evaluation
             val_X = self.train_X_identity
         else:
@@ -552,6 +443,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
             gc.collect()
 
         # build one time, then reset if needed
+
         if NO_AUX:
             # we could load with file name, then remove and save to new one
             h5_file = prefix + "_attention_lstm_NOAUX_" + f".hdf5"
@@ -564,6 +456,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
         starter_lr = params.get("starter_lr", STARTER_LEARNING_RATE)
         # model thing
         load_from_model = False
+
         if re_train or not os.path.isfile(h5_file):
             logger.debug(
                 f"re_train is {re_train}, file {h5_file} exists? {os.path.isfile(h5_file)}"
@@ -624,8 +517,10 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
         run_times = 0
         better_run_times = 0
         final_score = 0
+
         for fold in range(n_splits):
             # K.clear_session()  # so it will start over
+
             if fold > 0:
                 reinitLayers(model)
                 starter_lr = starter_lr / 8  # as lstm won't be re-initialized
@@ -641,7 +536,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
 
             if not predict_only:
                 prog_bar_logger = NBatchProgBarLogger(
-                    display_per_batches=int(1600000 / 20 / BATCH_SIZE),
+                    display_per_batches=int(len(train_X) / 10 / BATCH_SIZE),
                     early_stop=True,
                     patience_displays=3,
                 )
@@ -652,6 +547,13 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
                     # we also need to filter out the subgroup data (in outer layer now)
                     val_y = val_y >= 0.5
 
+                sample_weights = None
+
+                if train_y.shape[1] > 1:
+                    _train_y = train_y[:,0]
+                    sample_weights = train_y[:,1]
+                    train_y = _train_y
+
                 train_labels = train_y if NO_AUX else [train_y, train_y_aux]
                 val_labels = val_y if NO_AUX else [val_y, val_y_aux]
 
@@ -661,7 +563,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
                     # validation_split=val_split,
                     batch_size=BATCH_SIZE,
                     epochs=EPOCHS,
-                    # sample_weight=sample_weights,
+                    sample_weight=sample_weights,
                     # steps_per_epoch=int(len(self.train_X)*False95/BATCH_SIZE),
                     verbose=0,  # prog_bar_logger will handle log, so set to 0 here
                     # validation_data=(self.train_X[val_ind], self.train_y[val_ind]>0.5),
@@ -685,6 +587,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
                 pred, val_mask, run_times=run_times
             )
             improve_flag = False
+
             if final_score_this_run > final_score:
                 final_score = final_score_this_run
                 improve_flag = True
@@ -695,6 +598,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
                 # if final_train:
                 test_result = model.predict(self.to_predict_X_all, verbose=2)
                 run_times += 1
+
                 if NO_AUX:
                     test_result_column = np.array(test_result).ravel()
                 else:
@@ -716,6 +620,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
     def check_preds_in_val(self, pred, val_mask, run_times=0):
         if DEBUG_TRACE:
             set_trace()
+
         if not NO_AUX:
             preds = np.array(pred[0]).ravel()
         else:
@@ -726,6 +631,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
         # preds = 1 / (1+np.exp(-preds))
 
         self.train_df.loc[val_mask, "lstm"] = preds
+
         if True:
             self.train_df[d.VAL_ERR_COLUMN] = 0
             # self.train_df.loc[val_mask, 'lstm'] = preds
@@ -734,8 +640,10 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
             )
             dstr = self.target_analyzer.get_err_distribution(
                 self.train_df, val_mask)
+
             for k, v in dstr.items():
                 logger.debug(f"error info: {k}, {v[1]}")
+
         return self.calculate_metrics_and_print(
             filename_for_print=f"metrics_log_{run_times}.txt",
             validate_df_with_preds=kernel.train_df[val_mask],
@@ -755,6 +663,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
                 "prediction": predictions,
             }
         )
+
         if filename is not None:
             submission.to_csv(filename, index=False)
         else:
@@ -782,6 +691,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
         )  # just use sklearn split to get id and it is fine. For text thing,
         # just do 1 fold, later we can add them all back
         tr_ind, val_ind = splits[0]
+
         return subgroup_df.iloc[tr_ind].index, subgroup_df.iloc[val_ind].index
 
     def build_res_model(
@@ -836,12 +746,14 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
 
         # first: train only the top layers (which were randomly initialized)
         # i.e. freeze all convolutional InceptionV3 layers
+
         for layer in base_model.layers:
             layer.trainable = False
         # compile the model (should be done *after* setting layers to non-trainable)
         model = Model(inputs=base_model.input, outputs=result)
 
         model.compile(optimizer="adam", loss=loss, metrics=metrics)
+
         return model
 
     # TODO put to model Trainer (subclass)
@@ -873,6 +785,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
 
         for fold in range(n_splits):  # will need to do this later
             # K.clear_session()  # so it will start over todo fix K fold
+
             if use_split:
                 tr_ind, val_ind = splits[fold]
                 logger.info(
@@ -900,6 +813,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
             )
 
             # data thing
+
             if NO_AUX:
                 y_train = y[tr_ind]
                 y_val = y[val_ind]
@@ -948,10 +862,12 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
     def run_model(self, model, mode, X, y=None, params={}, n_splits=0):
         # checkpoint_predictions = []
         # weights = []
+
         if mode == "train":
             self.run_model_train(model, X, y, params, n_splits > 1, n_splits)
         elif mode == "predict":
             pred = model.predict(X, verbose=2, batch_size=BATCH_SIZE)
+
             return pred
 
         # if predict_ones_with_identity:
@@ -963,11 +879,13 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
         :param index: must be for the index of range 405130
         :return:
         """
+
         return self.train_X[index], self.train_y[index], self.train_y_aux[index]
 
     def locate_subgroup_index_in_np_train(self, subgroup):
         df = self.id_validate_df
         index = df[df[subgroup]].index
+
         return index
 
     def locate_subgroup_data_in_np_train(self, subgroup):
@@ -977,6 +895,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
         :return:
         """
         index = self.locate_subgroup_index_in_np_train(subgroup)
+
         return self.locate_data_in_np_train(index)
 
     def to_identity_index(self, index):
@@ -988,6 +907,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
         df = self.id_validate_df
 
         # selected the items
+
         return [df.index.get_loc(label) for label in index]
 
     def _get_identities(self):
@@ -998,6 +918,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
         """
         prefix = self.emb.BIN_FOLDER
         # if os.path.isfile(prefix+'train_df.pd'):
+
         if False:
             self.train_df = pickle.load(open(prefix + "train_df.pd", "rb"))
         else:
@@ -1026,6 +947,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
             self.train_mask[id_train_df_idx] = 1
 
             self.id_used_in_train = True
+
             for g in d.IDENTITY_COLUMNS:
                 # column to keep recored what data is used in training, used in data_prepare module...
                 self.train_df[g + "_in_train"] = 0.0
@@ -1079,6 +1001,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
 
             if balance_scheme_subgroups == "target_bucket_same_for_subgroups":
                 # compare with the background one, then change the weights to the same scale
+
                 for g, v in gs.items():
                     gs_weights[g] = ones_weights.copy()  # initial, ones
                     # v is the distribution for ONE subgroup for 0~1 11 target types
@@ -1086,12 +1009,14 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
                         background_target_ratios, np.array(
                             [dstr[2] for dstr in v])
                     )
+
                     for target_split_idx, ratio in enumerate(gs_weights_ratio[g]):
                         # [3] is the index
                         split_idx_in_df = v[target_split_idx][3]
                         gs_weights[g][split_idx_in_df] *= ratio
 
             # or 1->0.8 slop or 0.8->1, or y=-(x-1/2)^2+1/4; just test
+
             if balance_scheme_across_subgroups == "more_for_low_score":
                 subgroup_weights = {}
                 subgroup_weights["homosexual_gay_or_lesbian"] = 4
@@ -1103,8 +1028,10 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
     'male', 'female', 'homosexual_gay_or_lesbian', 'christian', 'jewish',
     'muslim', 'black', 'white', 'psychiatric_or_mental_illness'
                 """
+
                 for g in subgroup_weights.keys():
                     subgroup_dist = gs[g]
+
                     for dstr in subgroup_dist:
                         split_idx_in_df = dstr[3]
                         # the ones with identities will be added
@@ -1170,6 +1097,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
             if wanted_split_ratios is not None:
                 assert len(wanted_split_ratios) == len(
                     background_target_ratios)
+
                 for target_split_idx, ratio in enumerate(background_target_ratios):
                     idx_for_split = o[target_split_idx][3]
                     # 1/len(b_t_r) is what we want
@@ -1178,6 +1106,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
                     )
 
             sample_weights /= sample_weights.mean()  # normalize
+
             return sample_weights
 
         weights = add_weight(balance_scheme_subgroups)
@@ -1185,17 +1114,64 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
         return weights
 
     def prepare_train_labels(
-        self,
-        train_y_all,
-        train_mask,
-        custom_weights=False,
-        with_aux=False,
-        train_y_aux=None,
-        sample_weights=None,
-        fortify_subgroup=None,
+            self,
+            train_y_all,
+            train_mask,
+            custom_weights=False,
+            with_aux=False,
+            train_y_aux=None,
+            sample_weights=None,
+            fortify_subgroup=None,
     ):
         val_mask = np.invert(kernel.train_mask)  # this is whole train_mask
 
+        if fortify_subgroup is not None:
+            # only use the subgroup data
+            train_mask = train_mask & (
+                self.train_df[fortify_subgroup + "_in_train"] >= 0.5
+            )
+
+        train_X = kernel.train_X_all[train_mask]
+        val_X = kernel.train_X_all[val_mask]
+
+        if not custom_weights:
+            if with_aux:
+                return (
+                    train_X,
+                    val_X,
+                    train_y_all[train_mask],
+                    train_y_aux[train_mask],
+                    train_y_all[val_mask],
+                    train_y_aux[val_mask],
+                )
+            else:
+                return train_X, val_X, train_y_all[train_mask], train_y_all[val_mask]
+        else:
+            # credit to https://www.kaggle.com/tanreinama/simple-lstm-using-identity-parameters-solution
+
+            if sample_weights is None:
+                raise RuntimeError(
+                    "sample weights cannot be None if use custom_weights"
+                )
+            assert len(train_y_all) == len(sample_weights)
+
+            if with_aux:
+                return (
+                    train_X,
+                    val_X,
+                    np.vstack([train_y_all, sample_weights]).T[train_mask],
+                    train_y_aux[train_mask],
+                    train_y_all[val_mask],
+                    train_y_aux[val_mask],
+                )
+            else:
+                return (
+                    train_X,
+                    val_X,
+                    np.vstack([train_y_all, sample_weights]).T[train_mask],
+                    train_y_all[val_mask],
+                )
+
     def res_combine_pred_print_result(
         self, subgroup, y_pred, y_res_pred, idx_train, idx_val, detail=False
     ):
@@ -1255,6 +1231,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
 
         self.emb.read_train_test_df(train_only=True)
         self.load_identity_data_idx()
+
         if benchmark_base is None:
             benchmark_base = self.train_df.loc[self.identity_idx]
 
@@ -1266,10 +1243,12 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
         if model_name == d.MODEL_NAME:
             if preds is not None:
                 logger.debug(f"{model_name} result for {len(preds)} items:")
+
             if validate_df_with_preds is not None:
                 logger.debug(
                     f"{model_name} result for {len(validate_df_with_preds)} items:"
                 )
+
             if validate_df_with_preds is not None:
                 (
                     value,
@@ -1311,6 +1290,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
 
         # bias_metrics_df = pickle.load(open("bias_metrics", 'rb'))  # only the ones with identity is predicted
         # subgroup_distribution = pickle.load(open("subgroup_dist", 'rb'))  # only the ones with identity is predicted
+
         if not detail:
             return
 
@@ -1331,6 +1311,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
 
         def d_str(t):
             num, mean, std = t
+
             return f"{num}, {mean:.4}, {std:.4}"
 
         for d0 in subgroup_distribution:
@@ -1355,6 +1336,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
             )
 
         print("### bpsn auc", file=file_for_print)
+
         for d0 in subgroup_distribution:
             g = d0["subgroup"]
             m = "bpsn_auc"
@@ -1373,6 +1355,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
             )
 
         print("### bnsp auc", file=file_for_print)
+
         for d0 in subgroup_distribution:
             g = d0["subgroup"]
             m = "bnsp_auc"
@@ -1392,6 +1375,7 @@ BS {BATCH_SIZE}, NO_ID_IN_TRAIN {EXCLUDE_IDENTITY_IN_TRAIN}, EPOCHS {EPOCHS}, Y_
 
         print("### counts", file=file_for_print)
         # length thing
+
         for d0 in subgroup_distribution:
             g = d0["subgroup"]
             m = "subgroup_auc"
@@ -1465,12 +1449,13 @@ NEG_RATIO = 1 - 0.05897253769515213
 
 
 def main(argv):
-    args = parser.parse_args(argv[1:])
+    # args = parser.parse_args(argv[1:])
 
     global kernel
     logger.info("Will start load data")
     # kernel = KaggleKernel(action="convert_train_data")
     action = None
+
     if CONVERT_DATA:
         # action = TRAIN_DATA_EXCLUDE_IDENDITY_ONES
         # action = CONVERT_DATA_Y_NOT_BINARY
@@ -1484,6 +1469,7 @@ def main(argv):
             action = TRAIN_DATA_EXCLUDE_IDENDITY_ONES
     # if not (RESTART_TRAIN or RESTART_TRAIN_ID or RESTART_TRAIN_RES):
     #    action = DATA_ACTION_NO_NEED_LOAD_EMB_M  # loading model from h5 file, no need load emb matrix (save memory)
+
     if FINAL_SUBMIT:
         kernel = KaggleKernel(action=None)
     else:
@@ -1497,62 +1483,6 @@ def main(argv):
     # pred = pickle.load(open('predicts', 'rb'))
     prefix = kernel.emb.BIN_FOLDER
 
-    if IDENTITY_RUN:
-        # preds = np.where(preds >= 0.5, True, False) no need recording to API description, but why auc value can change?
-        for idtt in d.IDENTITY_COLUMNS:
-            # for idtt in ['male', 'female', 'homosexual_gay_or_lesbian', 'christian', 'jewish','muslim', 'black', 'white', 'psychiatric_or_mental_illness'][6:]:
-            if PRD_ONLY:
-                # to_predict = kernel.to_predict_X_all
-                to_predict_all = utils.get_obj_and_save("additional_data.bin")
-                len_all = len(to_predict_all)
-                one_len = int(len_all // 10)
-                id_additional_pred = []
-                for i in range(10):
-                    to_predict = kernel.emb._prepare_additional_data_pad(
-                        to_predict_all[one_len * i: one_len * (i + 1)]
-                    )
-
-                    set_trace()
-                    id_pred = kernel.run_identity_model(
-                        idtt,
-                        to_predict,
-                        None,
-                        params={  # TODO absctract and get params for running
-                            "prefix": prefix,
-                            "starter_lr": STARTER_LEARNING_RATE,
-                            "epochs": 8,
-                            "patience": 2,
-                            "lr_decay": 0.5,
-                            "validation_split": 0.05,
-                            "no_check_point": False,  # save check point or not
-                            "verbose": 2,
-                            "continue_train": False,  # just predict, do not train this time
-                            "predict_only": True,
-                        },
-                    )
-                    id_additional_pred = id_additional_pred + id_pred
-                    del to_predict
-                utils.dump_obj(id_additional_pred, f"{idtt}_additional_pred")
-            else:  # train
-                kernel.run_identity_model(
-                    idtt,
-                    kernel.train_X_identity,
-                    kernel.train_y_identity,
-                    params={
-                        "prefix": prefix,
-                        "starter_lr": STARTER_LEARNING_RATE,
-                        "epochs": 8,
-                        "patience": 2,
-                        "lr_decay": 0.5,
-                        "validation_split": 0.05,
-                        "no_check_point": False,  # save check point or not
-                        "verbose": 2,
-                        "continue_train": False,  # just predict, do not train this time
-                    },
-                )
-
-        return
-
     if TARGET_RUN == "res":
         # if not os.path.isfile('predicts'):
         preds = pickle.load(open("predicts", "rb"))
@@ -1561,6 +1491,7 @@ def main(argv):
 
     elif TARGET_RUN == "lstm":
         predict_only = PRD_ONLY
+
         if not TARGET_RUN_READ_RESULT:
             if ANA_RESULT:
                 # preds = pickle.load(open('predicts', 'rb'))
@@ -1578,6 +1509,7 @@ def main(argv):
                 val_X = None
                 train_y_aux = None
                 train_y = None
+                val_y_aux = None
 
                 if NO_AUX:
                     train_X, val_X, train_y, val_y = kernel.prepare_train_labels(
@@ -1619,7 +1551,7 @@ def main(argv):
                         "starter_lr": STARTER_LEARNING_RATE,
                         # 'sample_weights': sample_weights_train,
                         # train data with identities
-                        "train_data": (train_X, train_y),
+                        "train_data": (train_X[:10000], train_y[:10000,:]),
                         "train_y_aux": train_y_aux,
                         "val_y_aux": val_y_aux,
                         # train data with identities
@@ -1648,6 +1580,7 @@ def main(argv):
         # value, bias_metrics = kernel.evaluate_model(pred_target)
         # logger.info(value)
         # logger.info(f"\n{bias_metrics}")
+
     return
 
 
@@ -1664,6 +1597,7 @@ WEIGHT_TO_Y = True
 
 
 ANA_RESULT = False
+
 if os.path.isfile(".ana_result"):
     ANA_RESULT = True
     RESTART_TRAIN = False
